package io.gitlab.jfronny.inceptum.imgui.window.dialog;

import imgui.ImGui;
import io.gitlab.jfronny.commons.throwable.ThrowingConsumer;
import io.gitlab.jfronny.inceptum.launcher.LauncherEnv;
import io.gitlab.jfronny.inceptum.imgui.window.Window;
import io.gitlab.jfronny.inceptum.launcher.util.ProcessState;
import org.jetbrains.annotations.Nullable;

import java.util.concurrent.atomic.AtomicBoolean;

public class ProcessStateWatcherWindow extends Window {
    private final ProcessState state;
    private final Runnable cancel;
    private final AtomicBoolean canceled = new AtomicBoolean(false);
    private boolean finished;

    public ProcessStateWatcherWindow(String title, String errorMessage, ProcessState state, ThrowingConsumer<AtomicBoolean, ?> executor, @Nullable Runnable cancel) {
        super(title);
        this.state = state;
        this.cancel = cancel;
        new Thread(() -> {
            try {
                executor.accept(canceled);
                if (canceled.get() && cancel != null) cancel.run();
                finished = true;
            } catch (Throwable e) {
                LauncherEnv.showError(errorMessage, e);
            }
            close();
        }).start();
    }

    public ProcessState getState() {
        return state;
    }

    @Override
    public void draw() {
        ImGui.progressBar(state.getProgress());
        ImGui.textUnformatted(state.getCurrentStep());
        if (cancel != null && ImGui.button("Cancel")) {
            canceled.set(true);
        }
    }

    @Override
    public void close() {
        if (finished) {
            super.close();
        } else if (cancel != null) {
            canceled.set(true);
        }
    }
}
