package io.gitlab.jfronny.inceptum.imgui.window.edit;

import imgui.ImGui;
import io.gitlab.jfronny.commons.throwable.ThrowingBiConsumer;
import io.gitlab.jfronny.inceptum.common.R;
import io.gitlab.jfronny.inceptum.launcher.LauncherEnv;
import io.gitlab.jfronny.inceptum.imgui.GuiMain;
import io.gitlab.jfronny.inceptum.imgui.control.Tab;
import io.gitlab.jfronny.inceptum.imgui.window.dialog.ProcessStateWatcherWindow;
import io.gitlab.jfronny.inceptum.imgui.window.dialog.TextBoxWindow;
import io.gitlab.jfronny.inceptum.launcher.system.export.InstanceExporter;
import io.gitlab.jfronny.inceptum.launcher.util.ProcessState;
import org.lwjgl.PointerBuffer;
import org.lwjgl.system.MemoryStack;
import org.lwjgl.util.tinyfd.TinyFileDialogs;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Consumer;

public class ExportTab extends Tab {
    private final InstanceEditWindow window;

    public ExportTab(InstanceEditWindow window) {
        super("Export");
        this.window = window;
    }

    @Override
    protected void renderInner() {
        if (window.mds.isComplete()) {
            AtomicReference<String> cfVersion = new AtomicReference<>();
            exportVariant("CurseForge", "zip", it -> {
                GuiMain.WINDOWS.add(new TextBoxWindow("Version", "Please enter the version to list in the modpack file", "1.0", version -> {
                    cfVersion.set(version);
                    it.run();
                }, R::nop));
            }, (state, exportPath) -> {
                InstanceExporter.exportCurseZip(state, window.path, window.instance, window.mds, exportPath, cfVersion.get());
            });
            exportVariant("MultiMC", "zip", Runnable::run, (state, exportPath) -> {
                InstanceExporter.exportMultiMCZip(state, window.path, window.instance, window.mds, exportPath);
            });
        } else {
            ImGui.text("The mods directory scan must be completed.\nThe progress for this can be observed in the mods tab");
        }
    }

    private <T extends Throwable> void exportVariant(String name, String packExt, Consumer<Runnable> beforeSave, ThrowingBiConsumer<ProcessState, Path, T> export) {
        if (ImGui.button(name)) {
            beforeSave.accept(() -> {
                try (MemoryStack stack = MemoryStack.stackPush()) {
                    PointerBuffer aFilterPatterns = stack.mallocPointer(2);
                    aFilterPatterns.put(stack.UTF8("*." + packExt));
                    aFilterPatterns.flip();
                    String p = TinyFileDialogs.tinyfd_saveFileDialog("Export Pack", "", aFilterPatterns, name + " packs (*." + packExt + ")");
                    if (p != null) {
                        ProcessState state = new ProcessState(InstanceExporter.STEP_COUNT, "Initializing...");
                        GuiMain.open(new ProcessStateWatcherWindow("Exporting", "Could not export pack", state, cToken -> {
                            Path exportPath = Paths.get(p);
                            export.accept(state, exportPath);
                            LauncherEnv.showInfo(window.path.getFileName().toString() + " has been successfully exported to " + exportPath, "Successfully exported");
                        }, null));
                    }
                }
            });
        }
    }
}
