package io.gitlab.jfronny.inceptum.launcher.model.mojang;

import java.util.*;

@SuppressWarnings("MethodDoesntCallSuperMethod")
public class VersionInfo extends VersionsListInfo implements Cloneable {
    public Arguments arguments;
    public AssetIndex assetIndex;
    public String assets;
    public Downloads downloads;
    public JavaVersion javaVersion;
    public List<Library> libraries;
    public String mainClass;
    public String minecraftArguments;
    public Integer minimumLauncherVersion;

    @Override
    public VersionInfo clone() {
        VersionInfo clone = new VersionInfo();
        clone.arguments = arguments.clone();
        clone.assetIndex = assetIndex.clone();
        clone.assets = assets;
        clone.downloads = downloads.clone();
        clone.javaVersion = javaVersion.clone();
        clone.libraries = new LinkedList<>();
        for (Library library : libraries) clone.libraries.add(library.clone());
        clone.mainClass = mainClass;
        clone.minecraftArguments = minecraftArguments;
        clone.minimumLauncherVersion = minimumLauncherVersion;
        return clone;
    }

    public static class Arguments implements Cloneable {
        public List<MinecraftArgument> game;
        public List<MinecraftArgument> jvm;

        @Override
        public Arguments clone() {
            Arguments clone = new Arguments();
            clone.game = new LinkedList<>();
            for (MinecraftArgument argument : game) clone.game.add(argument.clone());
            clone.jvm = new LinkedList<>();
            for (MinecraftArgument argument : jvm) clone.jvm.add(argument.clone());
            return clone;
        }
    }

    public static class AssetIndex extends MojangFileDownload {
        public String id;
        public int totalSize;

        @Override
        public AssetIndex clone() {
            AssetIndex clone = new AssetIndex();
            clone.copyFrom(this);
            clone.id = id;
            clone.totalSize = totalSize;
            return clone;
        }
    }

    public static class Downloads implements Cloneable {
        public MojangFileDownload client;
        public MojangFileDownload client_mappings;
        public MojangFileDownload server;
        public MojangFileDownload server_mappings;

        @Override
        public Downloads clone() {
            Downloads clone = new Downloads();
            clone.client = client.clone();
            clone.client_mappings = client_mappings.clone();
            clone.server = server.clone();
            clone.server_mappings = server_mappings.clone();
            return clone;
        }
    }

    public static class JavaVersion implements Cloneable {
        public String component;
        public int majorVersion;

        @Override
        public JavaVersion clone() {
            JavaVersion clone = new JavaVersion();
            clone.component = component;
            clone.majorVersion = majorVersion;
            return clone;
        }
    }

    public static class Library implements Cloneable {
        public Downloads downloads;
        public String name;
        public Map<String, String> natives;
        public Rules rules;

        @Override
        public Library clone() {
            Library clone = new Library();
            clone.downloads = downloads.clone();
            clone.name = name;
            clone.natives = new LinkedHashMap<>();
            clone.natives.putAll(natives);
            clone.rules = rules.clone();
            return clone;
        }

        public static class Downloads implements Cloneable {
            public Artifact artifact;
            public Map<String, Artifact> classifiers;

            @Override
            public Downloads clone() {
                Downloads clone = new Downloads();
                clone.artifact = artifact.clone();
                clone.classifiers = new LinkedHashMap<>();
                classifiers.forEach((key, value) -> clone.classifiers.put(key, value.clone()));
                return clone;
            }

            public static class Artifact extends MojangFileDownload implements Cloneable {
                public String path;

                @Override
                public Artifact clone() {
                    Artifact clone = new Artifact();
                    clone.copyFrom(this);
                    clone.path = path;
                    return clone;
                }
            }
        }
    }
}
