package io.gitlab.jfronny.inceptum.launcher.util.ignore;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public class IgnoringWalk implements Iterator<Path> {
    public static Stream<Path> walk(Path repositoryRoot) throws IOException {
        return StreamSupport.stream(Spliterators.spliteratorUnknownSize(new IgnoringWalk(repositoryRoot), 0), false);
    }

    private static final String GITIGNORE = ".gitignore";
    private static final String GIT = ".git";

    private final Path ref;
    private final Queue<Path> toScan = new ArrayDeque<>(); // A queue of absolute Paths which must still be scanned/returned
    private final Map<Path, Ignore> ignores = new HashMap<>();
    private Path next;

    private IgnoringWalk(Path repositoryRoot) throws IOException {
        this.ref = repositoryRoot;
        enqueueContent(repositoryRoot);
    }

    @Override
    public Path next() {
        if (!hasNext()) throw new NoSuchElementException();
        Path ret = next;
        next = null;
        return ret;
    }

    @Override
    public boolean hasNext() {
        if (next != null) return true;
        next = toScan.poll();
        if (next == null) return false;
        if (Files.isDirectory(next)) {
            try {
                enqueueContent(next);
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        }
        return true;
    }

    private void enqueueContent(Path directory) throws IOException {
        Path ignorePath = directory.resolve(GITIGNORE);
        if (Files.exists(ignorePath)) {
            ignores.put(directory, new Ignore().add(Files.readAllLines(ignorePath)));
        }
        try (Stream<Path> files = Files.list(directory)) {
            for (Path path : files.toList()) {
                String fileName = path.getFileName().toString();
                if (!fileName.equals(GITIGNORE)
                        && !fileName.equals(GIT)
                        && !isIgnored(ref.relativize(path))) {
                    toScan.add(path);
                }
            }
        }
    }

    private boolean isIgnored(Path relativePath) {
        Path constructed = ref;
        for (Path segment : relativePath) {
            Ignore ignore = ignores.get(constructed);
            if (ignore != null && ignore.isIgnored(relativePath.toString()))
                return true;
            constructed = constructed.resolve(segment);
        }
        return false;
    }
}
