package io.gitlab.jfronny.inceptum;

import io.gitlab.jfronny.commons.ComparableVersion;
import io.gitlab.jfronny.commons.io.JFiles;
import io.gitlab.jfronny.inceptum.common.*;
import io.gitlab.jfronny.inceptum.common.model.inceptum.*;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.stream.Collectors;

public class Wrapper {
    public static void main(String[] args) throws IOException, ClassNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException, URISyntaxException {
        System.out.println("Inceptum Wrapper v" + BuildMetadata.VERSION);
        System.out.println("Loading from " + MetaHolder.BASE_PATH);
        InceptumEnvironmentInitializer.initialize();
        List<Path> classpath = getClasspath();
        if (!BuildMetadata.IS_RELEASE) {
            System.out.println("Using classpath: " + classpath.stream().map(Path::toString).collect(Collectors.joining("" + File.pathSeparator)));
        }
        System.out.println("Starting Inceptum ClassLoader");
        WrapperStrap.bootstrap(classpath,
                "io.gitlab.jfronny.inceptum.Inceptum",
                args);
    }

    private static List<Path> getClasspath() throws IOException, URISyntaxException {
        if (!Files.exists(MetaHolder.WRAPPER_CONFIG_PATH)) {
            UpdateMetadata update = Updater.check(UpdateChannel.Stable, new ComparableVersion("0"), R::nop);
            if (update == null) {
                throw new FileNotFoundException("Could not identify a valid inceptum version. Are you connected to the internet?");
            }
            Updater.update(update, false);
            if (!Files.exists(MetaHolder.WRAPPER_CONFIG_PATH))
                throw new FileNotFoundException("Something went wrong while downloading the latest version.");
        }
        return Updater.getLaunchClasspath(JFiles.readObject(MetaHolder.WRAPPER_CONFIG_PATH, WrapperConfig.class));
    }
}
